#!/usr/bin/python

"""
	Offers multiple GET with pattern support for ObexFTP

	Author:  Robin Rawson-Tetley
	Date:    7th September, 2006
	License: GPL v2
	
	Usage:
		obexmget [-Bchannel] [-Ppattern] -b<bdaddr> -d<directory>

"""

import os
import sys
import xml.sax.handler

channel=""	# SDP channel OBEX is on for target device
pattern="*"	# Default pattern match to apply to filenames in GET
bdaddr=""	# The BD address of the device
dir=""		# The directory to retrieve from

def copyright():
	print "ObexMGet 1.0"
	print "Copyright(c)2006, R. Rawson-Tetley"
	print "THIS SOFTWARE COMES WITH ABSOLUTELY NO WARRANTY!"

def usage():
	print "Usage: obexmget [-Bchannel] [-Ppattern] -b<bdaddr> -d<directory>"
	print ""
	print "    -Bchannel The OBEX channel on the target device (ignore to detect)"
	print "    -Ppattern The file pattern match to use when retrieving files (* = All, "
	print "              OR grep style regular expression)"
	print "    -bbdaddr  The BD addr of the target device"
	print "    -ddir     The directory to download files from"
	print ""
	print "  Examples:"
	print "     obexmget -B12 \"-P.*.jpg\" -b 99:99:99:99:99 -dE:/Images"

copyright()

# Parse command line args
for arg in sys.argv:
	if arg.startswith("-B"):
		channel = arg + " "
	if arg.startswith("-b"):
		bdaddr = arg[2:]
	if arg.startswith("-P"):
		pattern = arg[2:]
	if arg.startswith("-d"):
		dir = arg[2:]

# Bail if we're missing a directory or bd addr
if bdaddr == "" or dir == "":
	usage()
	sys.exit(1)

# Add trailing slash to the dir
if not dir.endswith("/"): dir = dir + "/"

print "\nStarting OBEX MGet with params:"
print "channel: %s, bdaddr: %s, pattern: %s, dir: %s\n" % (channel, bdaddr, pattern, dir)

# Grab the directory listing first:
lscmd = "obexftp -b %s %s-l \"%s\" 2>&1" % (bdaddr, channel, dir);
print "Invoking: " + lscmd
ls = os.popen(lscmd, "r")
listing = ""
while 1:
	line = ls.readline()
	if not line: break
	# If the line isn't XML, ignore it
	if not line.strip().startswith("<"): continue
	# Ignore special tags that make sax explode
	if line.strip().startswith("<!"): continue
	if line.strip().startswith("<?"): continue
	# Last line has "done" with a weird char attached
	if line.strip().endswith("done"):
		line = line[0:line.rfind(">")+1]
	listing = listing + line 
ls.close()

# Write a temporary file with the XML 
# (TODO: Must be a better way of parsing this)
tmpfile = "/tmp/obexlist.txt"
f = open(tmpfile, "w")
f.write(listing)
f.close()

# Our list containing the matching files
files = []

# XML handler for parsing listing format
class LsHandler(xml.sax.handler.ContentHandler):
	def startElement(self, name, attributes):
		if name == "file":
			# Does this file fulfil our match?
			fileok = False
			# * pattern just does everything
			if pattern == "*": 
				fileok = True
			else:
				# Submit the filename/pattern to grep
				grs = os.popen("echo '%s' | grep '%s'" % (attributes['name'], pattern), "r")
				while 1:
					line = grs.readline()
					if not line:
						fileok = False
					else:
						# Grep produced something, it's ok
						fileok = True
					break
				grs.close()

			# Add it to our list
			if fileok:
				file = dir + attributes['name'];
				files.append(file)
				#print "Found file: " + file


# Parse our listing
parser = xml.sax.make_parser()
handler = LsHandler()
parser.setContentHandler(handler)
parser.parse(tmpfile)

# Delete the temp file
os.remove(tmpfile)

# Do the retrieval
num = 1
for i in files:
	print "Fetching (" + str(num) + "/" + str(len(files)) + "): " + i
	getcmd = "obexftp -b %s %s-g \"%s\"" % (bdaddr, channel, i);
	os.system(getcmd)
	num = num + 1

print "Done."
